      subroutine setsrc
!****************************************************************
!*
!*   Reads the source file
!*
!*      FORMAT:
!*         x y flux dx dy UNITS
!*      where flux is given in UNITS. The mass flow rate is
!*      distribuited among nodes so that the total mass
!*      is conserved 
!* 
!****************************************************************
     use io_parameters
     use master_variables
     implicit none
!
     logical            :: point_src 
     character(len=250) :: fname
     character(len=15 ) :: units
     integer  (kind=4 ) :: ix,iy,kp
     real     (kind=8 ) :: x,y,xs,ys,flux,area,dxs,dys,ups
     real(kind=8),allocatable :: mass(:,:)
!
!*** allocates and initializes
!
     allocate(mass(nx,ny))
     mass(1:nx,1:ny) = 0.
     up  (1:nx,1:ny) = 0.
!
!*** Opens the file 
!
     open(90,FILE=TRIM(fsrc),STATUS='unknown',ERR=100)
! 
!*** Reads the discrete values 
!
     do while(0.ne.1)
         read(90,*,END=1,ERR=101) xs,ys,flux,dxs,dys,units
!
!***     Distinguish between point and extended sources.
!***     1. Extended sources. Units of kg/m2/s   Iterpolation ensuring mass conservation
!***     2. Point    sources. Units of kg/s      Up velocity calculated directly 
!
         if(TRIM(units).eq.'KG_M2_SEC') then
            flux = flux
            point_src = .false.
         else if(TRIM(units).eq.'KG_SEC') then
             ups = flux/(rho1*dxs*dys)
             point_src = .true.
         else if(TRIM(units).eq.'GR_M2_SEC') then
             flux = flux/1d3 
             point_src = .false.
         else if(TRIM(units).eq.'GR_SEC') then
             ups = flux/(1d3*rho1*dxs*dys)
             point_src = .true.
         else if(TRIM(units).eq.'TN_M2_DAY') then
             flux = flux*1d3/864d2
             point_src = .false.
         else if(TRIM(units).eq.'TN_DAY') then
             ups = flux*1d3/(864d2*rho1*dxs*dys)
             point_src = .true.
         else if(TRIM(units).eq.'KG_M2_DAY') then
             flux = flux/864d2
             point_src = .false.
         else if(TRIM(units).eq.'KG_DAY') then
             ups = flux/(864d2*rho1*dxs*dys)
             point_src = .true.
         else if(TRIM(units).eq.'GR_M2_DAY') then
             flux = flux/(1d3*864d2)
             point_src = .false.
         else if(TRIM(units).eq.'GR_DAY') then
             ups = flux/(1d3*864d2*rho1*dxs*dys)
             point_src = .true.
         else if(TRIM(units).eq.'M_S') then
             ups = flux
             point_src = .true.
         else
             call runend(-1,'Incorrect units in source file'//TRIM(fsrc))
         end if
!
!***     POINT SOURCE
!
         if(point_src) then
              do iy = 1,ny
               y = y0 + (iy-1)*dy
              do ix = 1,nx
                    x = x0 + (ix-1)*dx
                    if((x.le.xs).and.(x+dx.gt.xs).and.  &
                      (y.le.ys).and.(y+dy.gt.ys)) then
                        up(ix,iy) = up(ix,iy) + ups      
                     end if
                end do
             end do
           else
!
!***        EXTENDED SOURCE case 1. Source area <= grid size. Mass goes to the closest node
!
            if((dxs.le.dx).or.(dys.le.dy)) then
! 
              do iy = 1,ny
                 y = y0 + (iy-1)*dy      
              do ix = 1,nx
                    x = x0 + (ix-1)*dx
                    if((x.le.xs).and.(x+dx.gt.xs).and.  &
                        (y.le.ys).and.(y+dy.gt.ys)) then
                        mass(ix,iy) = mass(ix,iy) + flux*dxs*dys      ! ks/s
                end if
                end do
                end do
!
            else
!
!***        EXTENDED SOURCE case 2. Source area > grid size. 
!
              kp = 0                    ! count the number of points
              do iy = 1,ny
                  y = y0 + (iy-1)*dy
              do ix = 1,nx
                    x = x0 + (ix-1)*dx
                    if((x.ge.xs-0.5*dxs).and.(x.lt.xs+0.5*dxs).and. &
                    (y.ge.ys-0.5*dys).and.(y.lt.ys+0.5*dys))  kp = kp+1
              end do
              end do
              do iy = 1,ny
               y = y0 + (iy-1)*dy
               do ix = 1,nx
                    x = x0 + (ix-1)*dx
                    if((x.ge.xs-0.5*dxs).and.(x.lt.xs+0.5*dxs).and. &
                        (y.ge.ys-0.5*dys).and.(y.lt.ys+0.5*dys))     &
                         mass(ix,iy) = mass(ix,iy) + flux*dxs*dys/kp     ! kg/s
               end do
              end do
! 
          end if
        end if
       end do
 1   continue
     close(90)
!
!*** Convert from mass to up (affects just extended sources)
!
     up(1:nx,1:ny) = up(1:nx,1:ny) + mass(1:nx,1:ny)/(rho1*dx*dy)     ! convert to m/s
     deallocate(mass)
!
!*** If necessary, outputs the source term
!
     if(out_src) then
     fname = TRIM(dump)//'source.grd'
     call wrigrid(fname,nx,ny,x0,y0,dx,dy,up)    
     end if
!
!*** Prints to the log file
!
     flux = SUM(up(1:nx,1:ny))
     area = dx*dy
!
     write(nlog,10) TRIM(fsrc),            & 
                  flux*rho1*area,     &
                  MAXVAL(up(1:nx,1:ny)),      &
                  MINVAL(up(1:nx,1:ny)),      &
                  SUM   (up(1:nx,1:ny))/(nx*ny)
 10   format(/,  &
        'SOURCE FILE READ' ,/,  &  
        '  File name             : ',a    ,/,  & 
        '  Domain mass (kg/s  )  : ',e12.4,/,  &
        '  Up Maximum (m/s)      : ',e12.4,/,  &
        '  Up Minimum (m/s)      : ',e12.4,/,  &
        '  Up Average (m/s)      : ',e12.4)
!
     return
!
!*** List of errors
!
 100 call runend(-1,'Error opening file'//TRIM(fsrc))
 101 call runend(-1,'Error reading file'//TRIM(fsrc))
!      
      return
      end subroutine setsrc
